@@ -10,11 +10,22 @@
   #if __GNUC__ == 4
     #if __GNUC_MINOR__ < 9
 namespace std {
+      #if defined(__clang__)
+    union max_align_t {
+        void *a;
+        long long b;
+        double c;
+        long double d;
+    };
+      #else
     typedef ::max_align_t max_align_t;
+      #endif
 }
     #endif
     #if __GNUC_MINOR__ < 8
-      #define alignas(X) __attribute__ ((aligned (X)))
+      #if !defined(__clang__)
+        #define alignas(X) __attribute__ ((aligned (X)))
+      #endif
 
 namespace std {
     template<typename T> struct is_trivially_destructible : has_trivial_destructor<T> {};
@@ -27,10 +38,10 @@ namespace std {
 
   #define RESTRICT __restrict__
 
-  #ifdef __GNUC_GNU_INLINE__
-    #define FORCE_INLINE(RET) inline RET __attribute__((gnu_inline,always_inline))
+  #if defined(__GNUC_GNU_INLINE__) && !defined(__clang__)
+    #define FORCE_INLINE inline __attribute__((gnu_inline,always_inline))
   #else
-    #define FORCE_INLINE(RET) inline RET __attribute__((always_inline))
+    #define FORCE_INLINE inline __attribute__((always_inline))
   #endif
 #else
   #define LIKELY(X) X
@@ -39,9 +50,9 @@ namespace std {
   #define RESTRICT
 
   #if defined(_MSC_VER)
-    #define FORCE_INLINE(RET) __forceinline RET
+    #define FORCE_INLINE __forceinline
   #else
-    #define FORCE_INLINE(RET) inline RET
+    #define FORCE_INLINE inline
   #endif
 #endif
 
